import 'package:args/command_runner.dart';
import 'package:dcli/dcli.dart';
import '../archivers/archiver.dart';
import '../archivers/archivers.dart';

class Expand extends Command<int> {
  Expand() {
    argParser
      ..addOption('to',
          abbr: 't',
          defaultsTo: '.',
          help: 'The directory to expand the archive into.')
      ..addOption('name',
          abbr: 'n',
          mandatory: true,
          help: 'The path to the archive, including its extension, '
              'that is to be expanded.')
      ..addFlag('overwrite',
          abbr: 'o',
          help: "If set and the 'to' directory exists its contents will be "
              'deleted and replaced with the contents of the archive')
      ..addFlag('verbose',
          abbr: 'v', hide: true, help: 'Turns on verbose logging');
  }

  @override
  String get description => 'Expands files from an archive file';

  @override
  String get name => 'expand';

  @override
  int run() {
    /// global options
    Settings().setVerbose(enabled: argResults!['verbose'] as bool);

    // command options
    final pathToTar = join(pwd, argResults!['name'] as String);
    final targetDir = argResults!['to'] as String;
    final overwrite = argResults!['overwrite'] as bool;

    if (!exists(pathToTar)) {
      printerr(red("The archive $pathToTar doesn't exists."));
      return 1;
    }

    if (canonicalize(targetDir) == canonicalize(dirname(pathToTar))) {
      printerr(red('The to directory must not be the home of the tar file'));
      return 1;
    }

    if (exists(targetDir)) {
      if (!isEmpty(targetDir)) {
        if (!overwrite) {
          printerr(
              red("The 'to' directory '${truepath(targetDir)}' contains files. "
                  'Delete the files first or use the --overwrite flag.'));
          return 1;
        } else {
          if (!confirm('Are you certain you want to delete and overwrite '
              '${truepath(targetDir)}?')) {
            printerr(orange('Expansion cancelled'));
            return 1;
          }
        }
        if (truepath(targetDir) == truepath(pwd)) {
          // we are expanding into the current directory so delete everything.
          find('*', includeHidden: true, types: [Find.file, Find.directory])
              .forEach((entry) {
            if (isFile(entry)) {
              delete(entry);
            }
            if (isDirectory(entry)) {
              deleteDir(entry);
            }
            if (isLink(entry)) {
              deleteSymlink(entry);
            }
          });
        } else {
          delete(targetDir);
        }
      }
    } else {
      createDir(targetDir);
    }

    final archiver = Archivers().determineArchiver(pathToTar);
    if (archiver == null) {
      printerr(
          red("The archive type '${extension(pathToTar)}' isn't supported."));
      return 1;
    }

    if (!archiver.canCompress) {
      printerr(red('The archiver for ${archiver.extension}} '
          'does not support compression'));
    }

    if (!archiver.isInstalled(Operation.expand)) {
      printerr(red('''
  The archiver for '${archiver.extension}' is supported but isn't currently installed.
  Install ${archiver.command(Operation.expand)} and try again'''));
      return 1;
    }

    archiver.expand(pathToArchive: pathToTar, workingDirectory: targetDir);

    return 0;
  }
}
